-- This is BrokerGarbage's Loot Manager - inspired by KarniCrap
-- It allows you to manage auto looting and dropping of "too many" items
_, BrokerGarbage_LootManager = ...

-- default saved variables
BrokerGarbage_LootManager.defaultGlobalSettings = {
	-- behavior
	restackInventory = true,
	autoLoot = false,
	autoLootSkinning = true,
	autoLootFishing = true,
	autoLootPickpocket = true,
	closeLootWindow = true,
	forceClear = false,
	autoConfirmBoP = false,
	useInCombat = false,
	tooFewSlots = 0,
	openContainers = true,
	openClams = true,
	warnLM = true,
	warnInvFull = false,
	-- semi-internals
	privateLootTimer = 4,
}
BrokerGarbage_LootManager.defaultLocalSettings = {
	-- behavior
	itemMinValue = 0,
	autoDestroy = false,
	autoDestroyInstant = false,
}

-- used to distinguish between raid loot and inventory loot
hooksecurefunc("UseContainerItem", function(...)
	BrokerGarbage.privateLoot = GetTime()
end)
-- register events
local frame = CreateFrame("Frame")
local function eventHandler(self, event, arg1, ...)
	if event == "ADDON_LOADED" and arg1 == "Broker_Garbage-LootManager" then
		BrokerGarbage_LootManager:CheckSettings()
		frame:UnregisterEvent("ADDON_LOADED")

	elseif event == "ITEM_PUSH" and BGLM_LocalDB.autoDestroy and BGLM_LocalDB.autoDestroyInstant then
		if BrokerGarbage.totalFreeSlots < BGLM_GlobalDB.tooFewSlots then
			BrokerGarbage_LootManager:TrimInventory(BGLM_GlobalDB.tooFewSlots - BrokerGarbage.totalFreeSlots)
		end

	elseif event == "UI_ERROR_MESSAGE" then
			if arg1 and BGLM_GlobalDB.warnInvFull and (arg1 == ERR_INV_FULL or arg1 == INVENTORY_FULL) then
			BrokerGarbage:Print(BrokerGarbage_LootManager.locale.errorInventoryFull)
		end

	elseif event == "LOOT_OPENED" then
		-- restack inventory
		if BGLM_GlobalDB.restackInventory then
			local numSlots = 0
			local justStacked = {}
			
			for container = 0, 4 do
				numSlots = GetContainerNumSlots(container)
				if numSlots then
					for slot = 1, numSlots do
						local itemID = GetContainerItemID(container,slot)
						if itemID and not BrokerGarbage:Find(justStacked, itemID) then
							BrokerGarbage:Restack(itemID)
							table.insert(justStacked, itemID)
						end
					end
				end
			end
		end
		
		-- looting
		local disable = BrokerGarbage.disableKey[BrokerGarbage:GetOption("disableKey", true)]
		if not (disable and disable()) and (not InCombatLockdown() or BGLM_GlobalDB.useInCombat) then
			if BrokerGarbage.currentRestackItems ~= nil then
				BrokerGarbage.afterRestack = function()
					securecall(BrokerGarbage.SelectiveLooting, arg1)
				end
			else
				securecall(BrokerGarbage.SelectiveLooting, arg1)
			end
		end

	--[[elseif event == "LOOT_CLOSED" then
		if BGLM_LocalDB.autoDestroy and BGLM_LocalDB.autoDestroyInstant then
			-- TODO: recreate enough bag space
		end]]--

	-- this only gets called for restacking
	elseif event == "ITEM_UNLOCKED" then
		if BrokerGarbage:RestackStep() then
			-- wait for next update
			BrokerGarbage:Debug("Still restacking...", BrokerGarbage.currentRestackItems)
		else	-- we're done
			frame:UnregisterEvent("ITEM_UNLOCKED")
			BrokerGarbage.currentRestackItems = nil
			BrokerGarbage:Debug("Unregistered ITEM_UNLOCKED")
			
			if BrokerGarbage.afterRestack ~= nil then
				BrokerGarbage:afterRestack()
				BrokerGarbage.afterRestack = nil
			end
		end
	end
end

frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("ITEM_PUSH")
frame:RegisterEvent("UI_ERROR_MESSAGE")
frame:RegisterEvent("LOOT_OPENED")
-- frame:RegisterEvent("LOOT_CLOSED")
frame:SetScript("OnEvent", eventHandler)

-- ---------------------------------------------------------
-- Helper functions
-- ---------------------------------------------------------
function BrokerGarbage_LootManager:SetMinValue(value)
	BGLM_LocalDB.itemMinValue = value
end

function BrokerGarbage_LootManager:SetMinSlots(value)
	BGLM_GlobalDB.tooFewSlots = value
end

-- create default settings if not existant
function BrokerGarbage_LootManager:CheckSettings()
	-- check for settings
	if not BGLM_GlobalDB then BGLM_GlobalDB = {} end
	for key, value in pairs(BrokerGarbage_LootManager.defaultGlobalSettings) do
		if BGLM_GlobalDB[key] == nil then
			BGLM_GlobalDB[key] = value
		end
	end
	
	if not BGLM_LocalDB then BGLM_LocalDB = {} end
	for key, value in pairs(BrokerGarbage_LootManager.defaultLocalSettings) do
		if BGLM_LocalDB[key] == nil then
			BGLM_LocalDB[key] = value
		end
	end
end

-- restacks items so when deleting you lose as few items as possible
function BrokerGarbage:Restack(itemID)
	if BrokerGarbage.currentRestackItems then
		tinsert(BrokerGarbage.currentRestackItems, itemID)
	else
		BrokerGarbage.currentRestackItems = { itemID }
		if BrokerGarbage:RestackStep() then
			-- wait for moved items
			frame:RegisterEvent("ITEM_UNLOCKED")
		else
			-- nothing to restack
			if BrokerGarbage.afterRestack ~= nil then
				BrokerGarbage:afterRestack()
				BrokerGarbage.afterRestack = nil
			end
		end
	end
end

local function NextRestackStep()
	-- go to next item if there is one
	tremove(BrokerGarbage.currentRestackItems, 1)
	if #(BrokerGarbage.currentRestackItems) <= 0 then
		BrokerGarbage.currentRestackItems = nil
		return false
	else
		return BrokerGarbage:RestackStep()
	end
end

-- move 1 item for restacking
function BrokerGarbage:RestackStep()
	if not BrokerGarbage.currentRestackItems then return false end
	local itemID = BrokerGarbage.currentRestackItems[1]
	if not itemID then return NextRestackStep() end

	local count = GetItemCount(itemID)
	if not count or count <= 1 then return NextRestackStep() end
	
	local locations = BrokerGarbage:FindSlotToDelete(itemID, true)
	local maxLoc = #locations
	if maxLoc <= 1 then
		return NextRestackStep()
	end -- we're done, nothing to restack
	
	if GetContainerItemInfo(locations[1].bag, locations[1].slot) then
		ClearCursor()
		securecall(PickupContainerItem, locations[1].bag, locations[1].slot)
		securecall(PickupContainerItem, locations[maxLoc].bag, locations[maxLoc].slot)
		
		BrokerGarbage:Debug("Restack from/to", locations[1].count, locations[maxLoc].count)
	end
	return true
end

-- calls restack and deletes as many items as needed
function BrokerGarbage:DeletePartialStack(itemID, num)
	local locations = BrokerGarbage:FindSlotToDelete(itemID)
	local maxStack = select(8, GetItemInfo(itemID))
	
	if GetContainerItemID(locations[1].bag, locations[1].slot) ~= itemID then
		BrokerGarbage:Print("Error! DeletePartialStack: This is not the item I expected.")
		return
	end
	
	securecall(SplitContainerItem, locations[1].bag, locations[1].slot, num)
	if CursorHasItem() then
		BrokerGarbage:Delete("cursor", num)
		BrokerGarbage:Debug("DeletePartialStack", select(2,GetItemInfo(itemID)), num, locations[1].bag, locations[1].slot)
	end
end

-- returns true if the requested mob is skinnable with our skill
function BrokerGarbage:CanSkin(mobLevel)
	local skinning = BrokerGarbage:GetProfessionSkill(8613)
	if not skinning then return false end
	
	local maxLevel
	if skinning < 100 then 
		maxLevel = floor(skinning/10) + 10
	else 
		maxLevel = floor(skinning/5) 
	end
	
	return maxLevel >= mobLevel
end

-- determines if an item should be looted
function BrokerGarbage:IsInteresting(itemLink)
	local itemID = BrokerGarbage:GetItemID(itemLink)
	local alwaysLoot = BGLM_GlobalDB.forceClear or false
	
	-- items we don't want
	local negativeList = BrokerGarbage:JoinTables(BrokerGarbage:GetOption("include"))
	if not itemID or negativeList[itemID] then
		return false, false
	elseif BrokerGarbage.PT then
		-- check if the item belongs to a category
		local inCategory
		for setName,_ in pairs(negativeList) do
			if type(setName) == "string" then
				_, inCategory = BrokerGarbage.PT:ItemInSet(itemID, setName)
			end
			if inCategory then 
				return false, alwaysLoot
			end
		end
	end
	
	-- items we always want
	local positiveList = BrokerGarbage:JoinTables(BrokerGarbage:GetOption("exclude"))
	if positiveList[itemID] then
		alwaysLoot = true
	elseif BrokerGarbage.PT then
		-- check if the item belongs to a category
		local inCategory
		for setName,_ in pairs(positiveList) do
			if type(setName) == "string" then
				_, inCategory = BrokerGarbage.PT:ItemInSet(itemID, setName)
			end
			if inCategory then 
				alwaysLoot = true
			end
		end
	end
	
	if select(6,GetItemInfo(itemLink)) == select(12, GetAuctionItemClasses()) then	-- Quest Item
		alwaysLoot = true
	end
	
	-- items we don't care about
	return true, alwaysLoot
end

-- hook UpdateButton function for non-autoloot
function BrokerGarbage.UpdateLootFrame(index)
	if not index then return end
	local slot = (LOOTFRAME_NUMBUTTONS * (LootFrame.page - 1)) + index
	itemLink = GetLootSlotLink(slot)
	if not itemLink then return end
	
	local isInteresting, alwaysLoot = BrokerGarbage:IsInteresting(itemLink)
	if isInteresting or alwaysLoot then
		_G["LootButton"..index.."IconTexture"]:SetDesaturated(false)
		_G["LootButton"..index.."IconTexture"]:SetAlpha(1)
	else
		_G["LootButton"..index.."IconTexture"]:SetDesaturated(true)
		_G["LootButton"..index.."IconTexture"]:SetAlpha(0.5)
	end
end
hooksecurefunc("LootFrame_UpdateButton", BrokerGarbage.UpdateLootFrame)

-- ---------------------------------------------------------
-- lootmanager functionality from here on
-- ---------------------------------------------------------
-- for use in CHAT_MSG_LOOT event - destroys watched items as needed; UNUSED
function BrokerGarbage_LootManager:AutoDestroy()
	local location = {}
	local itemLink
	
	for itemID, maxCount in pairs(BrokerGarbage:JoinTables(BrokerGarbage:GetOption("include"))) do
		count = 0
		if type(itemID) == "number" and type(maxCount) == "number" then
			BrokerGarbage:Debug(itemID, maxCount)
			-- delete excess items
			location = BrokerGarbage:FindSlotToDelete(itemID)
			for i = #location, 1, -1 do
				if count >= maxCount then
					itemLink = select(2, GetItemInfo(itemID))
					BrokerGarbage:Delete(itemLink, {location[i].bag, location[i].slot})
				else
					count = count + location[i].count
				end
			end
		end
	end
end

function BrokerGarbage_LootManager:TrimInventory(emtySlotNum)
	if not emtySlotNum then return end
	for i = 1, emtySlotNum do
		local deleteThis = BrokerGarbage.cheapestItems[i]
		if not deleteThis then
			BrokerGarbage:Print("Error! I tried to make space but there is nothing left for me to delete!")
			return
		end
		BrokerGarbage:Delete(deleteThis)
	end
end

-- for use in LOOT_OPENED event
function BrokerGarbage.SelectiveLooting(autoloot)	-- jwehgH"G$(&/&§$/!!" stupid . vs. : notation
	if InCombatLockdown() and not BGLM_GlobalDB.useInCombat then return end
	
	if BrokerGarbage.privateLoot then
		if GetTime() - BrokerGarbage.privateLoot <= BGLM_GlobalDB.privateLootTimer then
			BrokerGarbage:Debug("Item is private loot")
		else
			BrokerGarbage.privateLoot = nil	-- reset, data is too old
		end
	end
	
	local autoLoot = autoloot ~= 0 or BGLM_GlobalDB.autoLoot
	local prepareSkinning = BGLM_GlobalDB.autoLootSkinning and UnitExists("target") and UnitIsDead("target")
		and UnitCreatureType("target") == BrokerGarbage_LootManager.locale.CreatureTypeBeast
		and BrokerGarbage:CanSkin(UnitLevel("target"))
	
	if autoLoot
		or (BGLM_GlobalDB.autoLootPickpocket and BrokerGarbage.playerClass == "ROGUE" and IsStealthed()) 
		or (BGLM_GlobalDB.autoLootFishing and IsFishingLoot())
		or prepareSkinning then
		BrokerGarbage:Debug("SelectiveLooting: Check passed, figure out what to do. AutoLoot:", autoloot)
		
		local close = true
		BrokerGarbage:Debug("close initialized: true")
		for slot = 1, GetNumLootItems() do
			local lootAction
			local _, _, quantity,  quality, locked = GetLootSlotInfo(slot)
			local itemLink = GetLootSlotLink(slot)
			local itemID = BrokerGarbage:GetItemID(itemLink)
			local compareTo = BrokerGarbage.cheapestItems[1]
			
			if itemLink then	-- needed because faulty loot slots or slots that contain money break things
				local value = BrokerGarbage:GetItemValue(itemLink, quantity) or 0
				local isInteresting, alwaysLoot = BrokerGarbage:IsInteresting(itemLink)
				local maxStack = select(8, GetItemInfo(itemID))
				local inBags = mod(GetItemCount(itemID), maxStack)
				local stackOverflow = quantity + mod(inBags, maxStack) - maxStack
				local lootMethod, groupLM, raidLM = GetLootMethod()
				
				if isInteresting or alwaysLoot then
					if not alwaysLoot and value < BGLM_LocalDB.itemMinValue then
						BrokerGarbage:Print(format(BrokerGarbage_LootManager.locale.couldNotLootValue, itemLink))
						lootAction = "none"
					
					elseif BrokerGarbage.totalFreeSlots <= BGLM_GlobalDB.tooFewSlots then
						-- dropping low on bag space
						BrokerGarbage:Debug("Free bag space below minimum treshold! Thinking ...", itemLink)
						
						if inBags > 0 and stackOverflow <= 0 then
							-- delete nothing. this item fits without us doing anything
							BrokerGarbage:Debug("Item stacks, do nothing special", itemLink)
							lootAction = "take"
						
						elseif not alwaysLoot and BGLM_LocalDB.autoDestroy and stackOverflow > 0 and 
							(prepareSkinning or (compareTo and (BrokerGarbage:GetItemValue(itemLink, stackOverflow) or 0) < compareTo.value)) then
							-- delete partial stack. throw away partial stacks to squeeze in a little more
							BrokerGarbage:Debug("Item can be made to fit.", itemLink)
							lootAction = "deletePartial"
						
						elseif BGLM_LocalDB.autoDestroy and 
							(((alwaysLoot or prepareSkinning) and compareTo) or 
							(compareTo and value > compareTo.value)) then
							-- delete only if it's worth more, if it's an item we really need or if we want to skin the mob
							BrokerGarbage:Debug("Deleting item", compareTo.itemLink, "to make room for", itemLink)
							lootAction = "delete"
						
						elseif not alwaysLoot and compareTo and value <= compareTo.value then
							BrokerGarbage:Debug("Taking this item by throwing away stuff would make us loose money.", itemLink)
							BrokerGarbage:Print(format(BrokerGarbage_LootManager.locale.couldNotLootCompareValue, itemLink))
							lootAction = "none"
						else
							-- we'd like to take the item but have no bag space (and can't make any)
							close = false
							if BrokerGarbage.privateLoot or lootMethod == "freeforall" or quality < GetLootThreshold() 
								or (GetNumPartyMembers() == 0 and GetNumRaidMembers() == 0) then
								-- we should be able to loot. if we can't it's because the inventory is full
								BrokerGarbage:Print(format(BrokerGarbage_LootManager.locale.couldNotLootSpace, itemLink))
							end
							lootAction = "none"
						end
					else
						lootAction = "take"
					end
				else
					-- item is on junk list
					BrokerGarbage:Print(format(BrokerGarbage_LootManager.locale.couldNotLootBlacklist, itemLink))
                    lootAction = "none"
				end
				
				-- last update & starting delete actions if needed
				if lootAction ~= "none" and locked and quality < GetLootThreshold() then
					-- we should probably be able to loot this, but something went wrong
					BrokerGarbage:Print(format(BrokerGarbage_LootManager.locale.couldNotLootLocked, itemLink))
                    lootAction = "none"
					close = false
					
				elseif lootMethod == "master" and quality >= GetLootThreshold() and
                    ((GetNumRaidMembers() > 1 and UnitIsUnit("raid"..raidLM, "player")) or 
                    (GetNumPartyMembers() > 0 and groupLM == 0)) then
                    -- we have loot master messages enabled and are loot master
                    if BGLM_GlobalDB.warnLM then
						BrokerGarbage:Print(format(BrokerGarbage_LootManager.locale.couldNotLootLM, itemLink))
					end
                    lootAction = "none"
					close = false
					break	-- prevent multiple messages when you're the LM
					
                elseif not BrokerGarbage.privateLoot and lootAction ~= "none" and 
					lootMethod ~= "freeforall" and quality >= GetLootThreshold() and 
					(GetNumPartyMembers() > 0 or GetNumRaidMembers() > 1) then
                    -- item is above the group's loot treshold and we're in a group
                    BrokerGarbage:Debug("Item is above loot treshold. Leave it for the user to decide.")
                    lootAction = "none"
					close = false

                elseif lootAction == "deletePartial" then
                    BrokerGarbage:DeletePartialStack(itemID, stackOverflow)
                    lootAction = "take"
                elseif lootAction == "delete" then
                    BrokerGarbage:Delete(compareTo)
                    lootAction = "take"
				end
			end
			
			if lootAction == "take" or LootSlotIsCoin(slot) then	-- finally, take it!	-- TODO: maybe add LootSlotIsCurrency
				BrokerGarbage:Debug("Taking item", itemLink or "Money")
				LootSlot(slot)
				if BrokerGarbage.privateLoot or BGLM_GlobalDB.autoConfirmBoP then
					BrokerGarbage:Debug("Confirming loot")
					ConfirmLootSlot(slot)
				end
			end
		end
		if close and BGLM_GlobalDB.closeLootWindow and (not IsFishingLoot() or not IsAddOnLoaded("FishingBuddy")) then
			CloseLoot()
		end
	end	-- TODO: maybe consider LOOT_SLOT_CLEARED event ... or not.
	BrokerGarbage.privateLoot = nil		-- if we used this, reset; if we didn't we don't need its value any more anyway
end